@@ -45,7 +45,9 @@ def svg_2d(chunks, offset=(0, 0), skew=(0, 0), size=200, sizes=None):
     sizes = sizes or draw_sizes(shape, size=size)
     y, x = grid_points(chunks, sizes)
 
-    lines, (min_x, max_x, min_y, max_y) = svg_grid(x, y, offset=offset, skew=skew)
+    lines, (min_x, max_x, min_y, max_y) = svg_grid(
+        x, y, offset=offset, skew=skew, size=size
+    )
 
     header = (
         '<svg width="%d" height="%d" style="stroke:rgb(0,0,0);stroke-width:1" >\n'
@@ -77,12 +79,14 @@ def svg_3d(chunks, size=200, sizes=None, offset=(0, 0)):
     ox, oy = offset
 
     xy, (mnx, mxx, mny, mxy) = svg_grid(
-        x / 1.7, y, offset=(ox + 10, oy + 0), skew=(1, 0)
+        x / 1.7, y, offset=(ox + 10, oy + 0), skew=(1, 0), size=size
     )
 
-    zx, (_, _, _, max_x) = svg_grid(z, x / 1.7, offset=(ox + 10, oy + 0), skew=(0, 1))
+    zx, (_, _, _, max_x) = svg_grid(
+        z, x / 1.7, offset=(ox + 10, oy + 0), skew=(0, 1), size=size
+    )
     zy, (min_z, max_z, min_y, max_y) = svg_grid(
-        z, y, offset=(ox + max_x + 10, oy + max_x), skew=(0, 0)
+        z, y, offset=(ox + max_x + 10, oy + max_x), skew=(0, 0), size=size
     )
 
     header = (
@@ -160,7 +164,7 @@ def svg_nd(chunks, size=200):
     return header + "\n\n".join(out) + footer
 
 
-def svg_lines(x1, y1, x2, y2):
+def svg_lines(x1, y1, x2, y2, max_n=20):
     """Convert points into lines of text for an SVG plot
 
     Examples
@@ -170,9 +174,15 @@ def svg_lines(x1, y1, x2, y2):
      '  <line x1="1" y1="0" x2="11" y2="1" style="stroke-width:2" />']
     """
     n = len(x1)
+
+    if n > max_n:
+        indices = np.linspace(0, n - 1, max_n, dtype="int")
+    else:
+        indices = range(n)
+
     lines = [
         '  <line x1="%d" y1="%d" x2="%d" y2="%d" />' % (x1[i], y1[i], x2[i], y2[i])
-        for i in range(n)
+        for i in indices
     ]
 
     lines[0] = lines[0].replace(" /", ' style="stroke-width:2" /')
@@ -180,7 +190,7 @@ def svg_lines(x1, y1, x2, y2):
     return lines
 
 
-def svg_grid(x, y, offset=(0, 0), skew=(0, 0)):
+def svg_grid(x, y, offset=(0, 0), skew=(0, 0), size=200):
     """Create lines of SVG text that show a grid
 
     Parameters
@@ -207,8 +217,9 @@ def svg_grid(x, y, offset=(0, 0), skew=(0, 0)):
     min_y = min(y1.min(), y2.min())
     max_x = max(x1.max(), x2.max())
     max_y = max(y1.max(), y2.max())
+    max_n = size // 6
 
-    h_lines = ["", "  <!-- Horizontal lines -->"] + svg_lines(x1, y1, x2, y2)
+    h_lines = ["", "  <!-- Horizontal lines -->"] + svg_lines(x1, y1, x2, y2, max_n)
 
     # Vertical lines
     x1 = x + offset[0]
@@ -222,13 +233,14 @@ def svg_grid(x, y, offset=(0, 0), skew=(0, 0)):
     if skew[1]:
         x2 += skew[1] * y.max()
 
-    v_lines = ["", "  <!-- Vertical lines -->"] + svg_lines(x1, y1, x2, y2)
+    v_lines = ["", "  <!-- Vertical lines -->"] + svg_lines(x1, y1, x2, y2, max_n)
 
+    color = "ECB172" if len(x) < max_n and len(y) < max_n else "8B4903"
+    corners = f"{x1[0]},{y1[0]} {x1[-1]},{y1[-1]} {x2[-1]},{y2[-1]} {x2[0]},{y2[0]}"
     rect = [
         "",
         "  <!-- Colored Rectangle -->",
-        '  <polygon points="%f,%f %f,%f %f,%f %f,%f" style="fill:#ECB172A0;stroke-width:0"/>'
-        % (x1[0], y1[0], x1[-1], y1[-1], x2[-1], y2[-1], x2[0], y2[0]),
+        f'  <polygon points="{corners}" style="fill:#{color}A0;stroke-width:0"/>',
     ]
 
     return h_lines + v_lines + rect, (min_x, max_x, min_y, max_y)
@@ -245,7 +257,7 @@ def grid_points(chunks, sizes):
 
 
 def draw_sizes(shape, size=200):
-    """ Get size in pixels for all dimensions """
+    """Get size in pixels for all dimensions"""
     mx = max(shape)
     ratios = [mx / max(0.1, d) for d in shape]
     ratios = [ratio_response(r) for r in ratios]
