@@ -760,12 +760,19 @@ def _get_symmat(size):
     return lA.dot(lA.T)
 
 
-@pytest.mark.xfail(
-    parse_version(scipy.__version__) >= parse_version("1.9.0"),
-    reason="https://github.com/dask/dask/issues/9335",
-)
+# `sym_pos` kwarg was deprecated in scipy 1.9.0
+# ref: https://github.com/dask/dask/issues/9335
+def _scipy_linalg_solve(a, b, assume_a):
+    if parse_version(scipy.__version__) >= parse_version("1.9.0"):
+        return scipy.linalg.solve(a=a, b=b, assume_a=assume_a)
+    elif assume_a == "pos":
+        return scipy.linalg.solve(a=a, b=b, sym_pos=True)
+    else:
+        return scipy.linalg.solve(a=a, b=b)
+
+
 @pytest.mark.parametrize(("shape", "chunk"), [(20, 10), (30, 6)])
-def test_solve_sym_pos(shape, chunk):
+def test_solve_assume_a(shape, chunk):
     np.random.seed(1)
 
     A = _get_symmat(shape)
@@ -775,26 +782,36 @@ def test_solve_sym_pos(shape, chunk):
     b = np.random.randint(1, 10, shape)
     db = da.from_array(b, chunk)
 
-    res = da.linalg.solve(dA, db, sym_pos=True)
-    assert_eq(res, scipy.linalg.solve(A, b, sym_pos=True), check_graph=False)
+    res = da.linalg.solve(dA, db, assume_a="pos")
+    assert_eq(res, _scipy_linalg_solve(A, b, assume_a="pos"), check_graph=False)
     assert_eq(dA.dot(res), b.astype(float), check_graph=False)
 
     # tall-and-skinny matrix
     b = np.random.randint(1, 10, (shape, 5))
     db = da.from_array(b, (chunk, 5))
 
-    res = da.linalg.solve(dA, db, sym_pos=True)
-    assert_eq(res, scipy.linalg.solve(A, b, sym_pos=True), check_graph=False)
+    res = da.linalg.solve(dA, db, assume_a="pos")
+    assert_eq(res, _scipy_linalg_solve(A, b, assume_a="pos"), check_graph=False)
     assert_eq(dA.dot(res), b.astype(float), check_graph=False)
 
     # matrix
     b = np.random.randint(1, 10, (shape, shape))
     db = da.from_array(b, (chunk, chunk))
 
-    res = da.linalg.solve(dA, db, sym_pos=True)
-    assert_eq(res, scipy.linalg.solve(A, b, sym_pos=True), check_graph=False)
+    res = da.linalg.solve(dA, db, assume_a="pos")
+    assert_eq(res, _scipy_linalg_solve(A, b, assume_a="pos"), check_graph=False)
     assert_eq(dA.dot(res), b.astype(float), check_graph=False)
 
+    with pytest.raises(FutureWarning, match="sym_pos keyword is deprecated"):
+        res = da.linalg.solve(dA, db, sym_pos=True)
+        assert_eq(res, _scipy_linalg_solve(A, b, assume_a="pos"), check_graph=False)
+        assert_eq(dA.dot(res), b.astype(float), check_graph=False)
+
+    with pytest.raises(FutureWarning, match="sym_pos keyword is deprecated"):
+        res = da.linalg.solve(dA, db, sym_pos=False)
+        assert_eq(res, _scipy_linalg_solve(A, b, assume_a="gen"), check_graph=False)
+        assert_eq(dA.dot(res), b.astype(float), check_graph=False)
+
 
 @pytest.mark.parametrize(("shape", "chunk"), [(20, 10), (12, 3), (30, 3), (30, 6)])
 def test_cholesky(shape, chunk):
