@@ -2,11 +2,10 @@ from dask.dataframe.io import demo
 from dask.dataframe.io.csv import read_csv, read_fwf, read_table, to_csv
 from dask.dataframe.io.hdf import read_hdf, to_hdf
 from dask.dataframe.io.io import (
-    dataframe_from_ctable,
     from_array,
-    from_bcolz,
     from_dask_array,
     from_delayed,
+    from_dict,
     from_map,
     from_pandas,
     to_bag,
