@@ -241,7 +241,7 @@ def test_to_hdf_modes_multiple_nodes():
         a.to_hdf(fn, "/data2")
         a.to_hdf(fn, "/data*", mode="a")
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # overwriting a file with a single partition
     a = dd.from_pandas(df, 1)
@@ -257,7 +257,7 @@ def test_to_hdf_modes_multiple_nodes():
         a.to_hdf(fn, "/data2")
         a.to_hdf(fn, "/data*", mode="a")
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # overwriting a file with two partitions
     a = dd.from_pandas(df, 2)
@@ -274,7 +274,7 @@ def test_to_hdf_modes_multiple_nodes():
         a.to_hdf(fn, "/data2")
         a.to_hdf(fn, "/data*", mode="a", append=False)
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
 
 def test_to_hdf_modes_multiple_files():
@@ -290,7 +290,7 @@ def test_to_hdf_modes_multiple_files():
         a.to_hdf(os.path.join(dn, "data2"), "/data")
         a.to_hdf(fn, "/data", mode="a")
         out = dd.read_hdf(fn, "/data*")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # appending two partitions to existing data
     a = dd.from_pandas(df, 2)
@@ -299,7 +299,7 @@ def test_to_hdf_modes_multiple_files():
         a.to_hdf(os.path.join(dn, "data2"), "/data")
         a.to_hdf(fn, "/data", mode="a")
         out = dd.read_hdf(fn, "/data")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
     # overwriting a file with two partitions
     a = dd.from_pandas(df, 2)
@@ -317,7 +317,7 @@ def test_to_hdf_modes_multiple_files():
         a.to_hdf(os.path.join(dn, "data1"), "/data")
         a.to_hdf(fn, "/data", mode="a", append=False)
         out = dd.read_hdf(fn, "/data")
-        assert_eq(df.append(df), out)
+        assert_eq(dd.concat([df, df]), out)
 
 
 def test_to_hdf_link_optimizations():
@@ -608,14 +608,14 @@ def test_to_fmt_warns():
 
     # testing warning when breaking order
     with tmpfile("h5") as fn:
-        with pytest.warns(None):
+        with pytest.warns(
+            UserWarning, match="To preserve order between partitions name_function"
+        ):
             a.to_hdf(fn, "/data*", name_function=str)
 
-    # testing warning when breaking order
     with tmpdir() as dn:
-        with pytest.warns(None):
-            fn = os.path.join(dn, "data_*.csv")
-            a.to_csv(fn, name_function=str)
+        fn = os.path.join(dn, "data_*.csv")
+        a.to_csv(fn, name_function=str)
 
 
 @pytest.mark.parametrize(
@@ -908,3 +908,14 @@ def test_hdf_nonpandas_keys():
         dd.read_hdf(path, "/group/table2")
         dd.read_hdf(path, "/group/table3")
         dd.read_hdf(path, "/bar")
+
+
+def test_hdf_empty_dataframe():
+    pytest.importorskip("tables")
+    # https://github.com/dask/dask/issues/8707
+    from dask.dataframe.io.hdf import dont_use_fixed_error_message
+
+    df = pd.DataFrame({"A": [], "B": []}, index=[])
+    df.to_hdf("data.h5", format="fixed", key="df", mode="w")
+    with pytest.raises(TypeError, match=dont_use_fixed_error_message):
+        dd.read_hdf("data.h5", "df")
