@@ -1,7 +1,4 @@
-import contextlib
-from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime
-from threading import Lock
 
 import numpy as np
 import pandas as pd
@@ -9,18 +6,18 @@ import pytest
 
 import dask.array as da
 import dask.dataframe as dd
+from dask import config
 from dask.blockwise import Blockwise
 from dask.dataframe._compat import tm
 from dask.dataframe.io.io import _meta_from_array
 from dask.dataframe.optimize import optimize
-from dask.dataframe.utils import assert_eq, is_categorical_dtype
+from dask.dataframe.utils import assert_eq
 from dask.delayed import Delayed, delayed
-from dask.utils import tmpfile
 from dask.utils_test import hlg_layer_topological
 
-####################
-# Arrays and BColz #
-####################
+##########
+# Arrays #
+##########
 
 
 def test_meta_from_array():
@@ -117,130 +114,6 @@ def test_from_array_with_record_dtype():
     assert (d.compute().to_records(index=False) == x).all()
 
 
-@contextlib.contextmanager
-def check_bcolz_deprecation_warning():
-    with pytest.warns(FutureWarning, match="bcolz was deprecated"):
-        yield
-
-
-def test_from_bcolz_multiple_threads():
-    bcolz = pytest.importorskip("bcolz")
-
-    def check(i):
-        t = bcolz.ctable(
-            [[1, 2, 3], [1.0, 2.0, 3.0], ["a", "b", "a"]], names=["x", "y", "a"]
-        )
-
-        d = dd.from_bcolz(t, chunksize=2)
-
-        assert d.npartitions == 2
-        assert is_categorical_dtype(d.dtypes["a"])
-        assert list(d.x.compute(scheduler="sync")) == [1, 2, 3]
-        assert list(d.a.compute(scheduler="sync")) == ["a", "b", "a"]
-
-        d = dd.from_bcolz(t, chunksize=2, index="x")
-
-        L = list(d.index.compute(scheduler="sync"))
-        assert L == [1, 2, 3] or L == [1, 3, 2]
-
-        # Names
-        assert sorted(dd.from_bcolz(t, chunksize=2).dask) == sorted(
-            dd.from_bcolz(t, chunksize=2).dask
-        )
-        assert sorted(dd.from_bcolz(t, chunksize=2).dask) != sorted(
-            dd.from_bcolz(t, chunksize=3).dask
-        )
-
-    with check_bcolz_deprecation_warning():
-        with ThreadPoolExecutor(5) as pool:
-            list(pool.map(check, range(5)))
-
-
-def test_from_bcolz():
-    bcolz = pytest.importorskip("bcolz")
-
-    t = bcolz.ctable(
-        [[1, 2, 3], [1.0, 2.0, 3.0], ["a", "b", "a"]], names=["x", "y", "a"]
-    )
-
-    with check_bcolz_deprecation_warning():
-        d = dd.from_bcolz(t, chunksize=2)
-        assert d.npartitions == 2
-        assert is_categorical_dtype(d.dtypes["a"])
-        assert list(d.x.compute(scheduler="sync")) == [1, 2, 3]
-        assert list(d.a.compute(scheduler="sync")) == ["a", "b", "a"]
-        L = list(d.index.compute(scheduler="sync"))
-        assert L == [0, 1, 2]
-
-        d = dd.from_bcolz(t, chunksize=2, index="x")
-        L = list(d.index.compute(scheduler="sync"))
-        assert L == [1, 2, 3] or L == [1, 3, 2]
-
-        # Names
-        assert sorted(dd.from_bcolz(t, chunksize=2).dask) == sorted(
-            dd.from_bcolz(t, chunksize=2).dask
-        )
-        assert sorted(dd.from_bcolz(t, chunksize=2).dask) != sorted(
-            dd.from_bcolz(t, chunksize=3).dask
-        )
-
-        dsk = dd.from_bcolz(t, chunksize=3).dask
-
-        t.append((4, 4.0, "b"))
-        t.flush()
-
-        assert sorted(dd.from_bcolz(t, chunksize=2).dask) != sorted(dsk)
-
-
-def test_from_bcolz_no_lock():
-    bcolz = pytest.importorskip("bcolz")
-    locktype = type(Lock())
-
-    t = bcolz.ctable(
-        [[1, 2, 3], [1.0, 2.0, 3.0], ["a", "b", "a"]], names=["x", "y", "a"], chunklen=2
-    )
-
-    with check_bcolz_deprecation_warning():
-        a = dd.from_bcolz(t, chunksize=2)
-        b = dd.from_bcolz(t, chunksize=2, lock=True)
-        c = dd.from_bcolz(t, chunksize=2, lock=False)
-
-    assert_eq(a, b)
-    assert_eq(a, c)
-
-    assert not any(isinstance(item, locktype) for v in c.dask.values() for item in v)
-
-
-def test_from_bcolz_filename():
-    bcolz = pytest.importorskip("bcolz")
-
-    with tmpfile(".bcolz") as fn:
-        t = bcolz.ctable(
-            [[1, 2, 3], [1.0, 2.0, 3.0], ["a", "b", "a"]],
-            names=["x", "y", "a"],
-            rootdir=fn,
-        )
-        t.flush()
-
-        with check_bcolz_deprecation_warning():
-            d = dd.from_bcolz(fn, chunksize=2)
-
-        assert list(d.x.compute()) == [1, 2, 3]
-
-
-def test_from_bcolz_column_order():
-    bcolz = pytest.importorskip("bcolz")
-
-    t = bcolz.ctable(
-        [[1, 2, 3], [1.0, 2.0, 3.0], ["a", "b", "a"]], names=["x", "y", "a"]
-    )
-
-    with check_bcolz_deprecation_warning():
-        df = dd.from_bcolz(t, chunksize=2)
-
-    assert list(df.loc[0].compute().columns) == ["x", "y", "a"]
-
-
 def test_from_pandas_dataframe():
     a = list("aaaaaaabbbbbbbbccccccc")
     df = pd.DataFrame(
@@ -377,6 +250,40 @@ def test_from_pandas_with_wrong_args():
         dd.from_pandas(df, chunksize=18.27)
 
 
+def test_from_pandas_chunksize_one():
+    # See: https://github.com/dask/dask/issues/9218
+    df = pd.DataFrame(np.random.randint(0, 10, size=(10, 4)), columns=list("ABCD"))
+    ddf = dd.from_pandas(df, chunksize=1)
+    num_rows = list(ddf.map_partitions(len).compute())
+    # chunksize=1 with range index should
+    # always have unit-length partitions
+    assert num_rows == [1] * 10
+
+
+@pytest.mark.parametrize(
+    "index",
+    [
+        ["A", "B", "C", "C", "C", "C", "C", "C"],
+        ["A", "B", "B", "B", "B", "B", "B", "C"],
+        ["A", "A", "A", "A", "A", "B", "B", "C"],
+    ],
+)
+def test_from_pandas_npartitions_duplicates(index):
+    df = pd.DataFrame({"a": range(8), "index": index}).set_index("index")
+    ddf = dd.from_pandas(df, npartitions=3)
+    assert ddf.divisions == ("A", "B", "C", "C")
+
+
+@pytest.mark.gpu
+def test_gpu_from_pandas_npartitions_duplicates():
+    cudf = pytest.importorskip("cudf")
+
+    index = ["A", "A", "A", "A", "A", "B", "B", "C"]
+    df = cudf.DataFrame({"a": range(8), "index": index}).set_index("index")
+    ddf = dd.from_pandas(df, npartitions=3)
+    assert ddf.divisions == ("A", "B", "C", "C")
+
+
 def test_DataFrame_from_dask_array():
     x = da.ones((10, 3), chunks=(4, 2))
     pdf = pd.DataFrame(np.ones((10, 3)), columns=["a", "b", "c"])
@@ -553,6 +460,20 @@ def test_from_dask_array_unknown_chunks():
     assert_eq(df, pd.DataFrame(dx.compute()), check_index=False)
 
 
+@pytest.mark.parametrize(
+    "chunksizes, expected_divisions",
+    [
+        pytest.param((1, 2, 3, 0), (0, 1, 3, 5, 5)),
+        pytest.param((0, 1, 2, 3), (0, 0, 1, 3, 5)),
+        pytest.param((1, 0, 2, 3), (0, 1, 1, 3, 5)),
+    ],
+)
+def test_from_dask_array_empty_chunks(chunksizes, expected_divisions):
+    monotonic_index = da.from_array(np.arange(6), chunks=chunksizes)
+    df = dd.from_dask_array(monotonic_index)
+    assert df.divisions == expected_divisions
+
+
 def test_from_dask_array_unknown_width_error():
     dsk = {("x", 0, 0): np.random.random((2, 3)), ("x", 1, 0): np.random.random((5, 3))}
     dx = da.Array(dsk, "x", ((np.nan, np.nan), (np.nan,)), np.float64)
@@ -1049,3 +970,20 @@ def test_from_map_column_projection():
     assert_eq(ddf["A"], expect["A"])
     assert set(projected) == {"A"}
     assert_eq(ddf, expect)
+
+
+@pytest.mark.gpu
+@pytest.mark.parametrize("backend", ["pandas", "cudf"])
+def test_from_dict_backends(backend):
+    _lib = pytest.importorskip(backend)
+    with config.set({"dataframe.backend": backend}):
+        data = {"a": [1, 2, 3, 4], "B": [10, 11, 12, 13]}
+        expected = _lib.DataFrame(data)
+
+        # Check dd.from_dict API
+        got = dd.from_dict(data, npartitions=2)
+        assert_eq(expected, got)
+
+        # Check from_dict classmethod
+        got_classmethod = got.from_dict(data, npartitions=2)
+        assert_eq(expected, got_classmethod)
