@@ -22,6 +22,7 @@ from dask.dataframe.core import (
     partitionwise_graph,
 )
 from dask.dataframe.multi import _maybe_align_partitions
+from dask.dataframe.utils import insert_meta_param_description
 from dask.delayed import unpack_collections
 from dask.highlevelgraph import HighLevelGraph
 from dask.utils import M, apply, derived_from, funcname, has_keyword
@@ -83,6 +84,7 @@ def overlap_chunk(func, before, after, *args, **kwargs):
     return out.iloc[before:-after]
 
 
+@insert_meta_param_description
 def map_overlap(
     func,
     df,
@@ -119,11 +121,12 @@ def map_overlap(
         Whether to enforce at runtime that the structure of the DataFrame
         produced by ``func`` actually matches the structure of ``meta``.
         This will rename and reorder columns for each partition,
-        and will raise an error if this doesn't work or types don't match.
-    before : int or timedelta
+        and will raise an error if this doesn't work,
+        but it won't raise if dtypes don't match.
+    before : int, timedelta or string timedelta
         The rows to prepend to partition ``i`` from the end of
         partition ``i - 1``.
-    after : int or timedelta
+    after : int, timedelta or string timedelta
         The rows to append to partition ``i`` from the beginning
         of partition ``i + 1``.
     transform_divisions : bool, default True
@@ -148,6 +151,11 @@ def map_overlap(
 
     dfs = [df for df in args if isinstance(df, _Frame)]
 
+    if isinstance(before, str):
+        before = pd.to_timedelta(before)
+    if isinstance(after, str):
+        after = pd.to_timedelta(after)
+
     if isinstance(before, datetime.timedelta) or isinstance(after, datetime.timedelta):
         if not is_datetime64_any_dtype(dfs[0].index._meta_nonempty.inferred_type):
             raise TypeError(
@@ -591,7 +599,9 @@ class Rolling:
         )
 
     @derived_from(pd_Rolling)
-    def aggregate(self, func, args=(), kwargs={}, **kwds):
+    def aggregate(self, func, args=(), kwargs=None, **kwds):
+        if kwargs is None:
+            kwargs = {}
         return self._call_method("agg", func, args=args, kwargs=kwargs, **kwds)
 
     agg = aggregate
