@@ -1,7 +1,7 @@
 import pytest
 
-from dask.context import globalmethod
 import dask
+from dask.context import globalmethod
 
 
 def test_with_get():
@@ -36,7 +36,7 @@ def bar():
 
 class Foo:
     @globalmethod(key="f")
-    def f():
+    def f():  # type: ignore
         return 1
 
     g = globalmethod(foo, key="g", falsey=bar)
