@@ -1,9 +1,11 @@
 import pickle
 import types
 from collections import namedtuple
+from dataclasses import dataclass, field
 from functools import partial
-from operator import add, setitem
+from operator import add, matmul, setitem
 from random import random
+from typing import NamedTuple
 
 import cloudpickle
 import pytest
@@ -13,13 +15,9 @@ import dask
 import dask.bag as db
 from dask import compute
 from dask.delayed import Delayed, delayed, to_task_dask
+from dask.highlevelgraph import HighLevelGraph
 from dask.utils_test import inc
 
-try:
-    from operator import matmul
-except ImportError:
-    matmul = None
-
 
 class Tuple:
     __dask_scheduler__ = staticmethod(dask.threaded.get)
@@ -56,11 +54,11 @@ def test_to_task_dask():
     assert task == (dict, [["b", 2], ["a", 1]]) or task == (dict, [["a", 1], ["b", 2]])
     assert dict(dask) == merge(a.dask, b.dask)
 
-    f = namedtuple("f", ["x", "y"])
-    x = f(1, 2)
+    f = namedtuple("f", ["a", "b", "c"])
+    x = f(a, b, 3)
     task, dask = to_task_dask(x)
-    assert task == x
-    assert dict(dask) == {}
+    assert task == (f, "a", "b", 3)
+    assert dict(dask) == merge(a.dask, b.dask)
 
     task, dask = to_task_dask(slice(a, b, 3))
     assert task == (slice, "a", "b", 3)
@@ -96,16 +94,12 @@ def test_delayed():
     assert a.key in b.dask
 
 
-def test_delayed_with_dataclass():
-    dataclasses = pytest.importorskip("dataclasses")
-
-    # Avoid @dataclass decorator as Python < 3.7 fail to interpret the type hints
-    ADataClass = dataclasses.make_dataclass(
-        "ADataClass", [("a", int), ("b", int, dataclasses.field(init=False))]
-    )
+def test_delayed_with_namedtuple():
+    class ANamedTuple(NamedTuple):
+        a: int
 
     literal = dask.delayed(3)
-    with_class = dask.delayed({"a": ADataClass(a=literal)})
+    with_class = dask.delayed({"a": ANamedTuple(a=literal)})
 
     def return_nested(obj):
         return obj["a"].a
@@ -115,6 +109,122 @@ def test_delayed_with_dataclass():
     assert final.compute() == 3
 
 
+@dataclass
+class ANonFrozenDataClass:
+    a: int
+
+
+@dataclass(frozen=True)
+class AFrozenDataClass:
+    a: int
+
+
+@pytest.mark.parametrize("cls", (ANonFrozenDataClass, AFrozenDataClass))
+def test_delayed_with_dataclass(cls):
+    literal = delayed(3)
+    with_class = delayed({"data": cls(a=literal)})
+
+    def return_nested(obj):
+        return obj["data"].a
+
+    final = delayed(return_nested)(with_class)
+
+    assert final.compute() == 3
+
+
+def test_delayed_with_dataclass_with_custom_init():
+    @dataclass()
+    class ADataClass:
+        a: int
+
+        def __init__(self, b: int):
+            self.a = b
+
+    literal = dask.delayed(3)
+
+    with pytest.raises(TypeError) as e:
+        dask.delayed({"data": ADataClass(b=literal)})
+
+    e.match(r"ADataClass")
+    e.match(r"custom __init__ is not supported")
+
+
+def test_delayed_with_dataclass_with_eager_custom_init():
+    @dataclass()
+    class ADataClass:
+        a: int
+
+        def __init__(self, b: int):
+            self.a = b
+
+    with_class = delayed({"data": ADataClass(b=3)})
+
+    def return_nested(obj):
+        return obj["data"].a
+
+    final = delayed(return_nested)(with_class)
+
+    assert final.compute() == 3
+
+
+def test_delayed_with_eager_dataclass_with_set_init_false_field():
+    @dataclass
+    class ADataClass:
+        a: int
+        b: int = field(init=False)
+
+    def prep_dataclass(a):
+        data = ADataClass(a=a)
+        data.b = 4
+        return data
+
+    with_class = delayed({"data": prep_dataclass(3)})
+
+    def return_nested(obj):
+        return obj["data"].a
+
+    final = delayed(return_nested)(with_class)
+
+    assert final.compute() == 3
+
+
+def test_delayed_with_dataclass_with_set_init_false_field():
+    @dataclass
+    class ADataClass:
+        a: int
+        b: int = field(init=False)
+
+    literal = dask.delayed(3)
+
+    def prep_dataclass(a):
+        data = ADataClass(a=a)
+        data.b = 4
+        return data
+
+    with pytest.raises(ValueError) as e:
+        dask.delayed(prep_dataclass(literal))
+
+    e.match(r"ADataClass")
+    e.match(r"`init=False` are not supported")
+
+
+def test_delayed_with_dataclass_with_unset_init_false_field():
+    @dataclass
+    class ADataClass:
+        a: int
+        b: int = field(init=False)
+
+    literal = dask.delayed(3)
+    with_class = delayed({"data": ADataClass(a=literal)})
+
+    def return_nested(obj):
+        return obj["data"].a
+
+    final = delayed(return_nested)(with_class)
+
+    assert final.compute() == 3
+
+
 def test_operators():
     a = delayed([1, 2, 3])
     assert a[0].compute() == 1
@@ -127,7 +237,7 @@ def test_operators():
     assert (1 + a).compute() == 11
     assert (a >> 1).compute() == 5
     assert (a > 2).compute()
-    assert (a ** 2).compute() == 100
+    assert (a**2).compute() == 100
 
     if matmul:
 
@@ -223,6 +333,9 @@ def test_delayed_optimize():
     (x2,) = dask.optimize(x)
     # Delayed's __dask_optimize__ culls out 'c'
     assert sorted(x2.dask.keys()) == ["a", "b"]
+    assert x2._layer != x2._key
+    # Optimize generates its own layer name, which doesn't match the key.
+    # `Delayed._rebuild` handles this.
 
 
 def test_lists():
@@ -539,11 +652,11 @@ def test_name_consistent_across_instances():
     func = delayed(identity, pure=True)
 
     data = {"x": 1, "y": 25, "z": [1, 2, 3]}
-    assert func(data)._key == "identity-02129ed1acaffa7039deee80c5da547c"
+    assert func(data)._key == "identity-4f318f3c27b869239e97c3ac07f7201a"
 
     data = {"x": 1, 1: "x"}
     assert func(data)._key == func(data)._key
-    assert func(1)._key == "identity-ca2fae46a3b938016331acac1908ae45"
+    assert func(1)._key == "identity-7258833899272585e16d0ec36b21a3de"
 
 
 def test_sensitive_to_partials():
@@ -701,14 +814,43 @@ def test_dask_layers():
     assert d2.dask.dependencies == {d1.key: set(), d2.key: {d1.key}}
     assert d2.__dask_layers__() == (d2.key,)
 
+    hlg = HighLevelGraph.from_collections("foo", {"alias": d2.key}, dependencies=[d2])
+    with pytest.raises(ValueError, match="not in"):
+        Delayed("alias", hlg)
 
-def test_dask_layers_to_delayed():
-    # da.Array.to_delayed squashes the dask graph and causes the layer name not to
-    # match the key
-    da = pytest.importorskip("dask.array")
-    d = da.ones(1).to_delayed()[0]
-    name = d.key[0]
-    assert d.key[1:] == (0,)
-    assert d.dask.layers.keys() == {"delayed-" + name}
-    assert d.dask.dependencies == {"delayed-" + name: set()}
-    assert d.__dask_layers__() == ("delayed-" + name,)
+    explicit = Delayed("alias", hlg, layer="foo")
+    assert explicit.__dask_layers__() == ("foo",)
+    explicit.dask.validate()
+
+
+def test_annotations_survive_optimization():
+    with dask.annotate(foo="bar"):
+        graph = HighLevelGraph.from_collections(
+            "b",
+            {"a": 1, "b": (inc, "a"), "c": (inc, "b")},
+            [],
+        )
+        d = Delayed("b", graph)
+
+    assert type(d.dask) is HighLevelGraph
+    assert len(d.dask.layers) == 1
+    assert len(d.dask.layers["b"]) == 3
+    assert d.dask.layers["b"].annotations == {"foo": "bar"}
+
+    # Ensure optimizing a Delayed object returns a HighLevelGraph
+    # and doesn't loose annotations
+    (d_opt,) = dask.optimize(d)
+    assert type(d_opt.dask) is HighLevelGraph
+    assert len(d_opt.dask.layers) == 1
+    assert len(d_opt.dask.layers["b"]) == 2  # c is culled
+    assert d_opt.dask.layers["b"].annotations == {"foo": "bar"}
+
+
+def test_delayed_function_attributes_forwarded():
+    @delayed
+    def add(x, y):
+        """This is a docstring"""
+        return x + y
+
+    assert add.__name__ == "add"
+    assert add.__doc__ == "This is a docstring"
